package examples;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.cassandra.thrift.Cassandra;
import org.apache.cassandra.thrift.Column;
import org.apache.cassandra.thrift.ColumnOrSuperColumn;
import org.apache.cassandra.thrift.ConsistencyLevel;
import org.apache.cassandra.thrift.InvalidRequestException;
import org.apache.cassandra.thrift.Mutation;
import org.apache.cassandra.thrift.TimedOutException;
import org.apache.cassandra.thrift.UnavailableException;
import org.apache.thrift.TException;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransportException;

/**
 * 複数のカラムをbatch_mutateを使ってまとめてインサート。
 * 
 */
public class SimpleMultipleColumnsInsert {

	public static final String KEYSPACE = "Keyspace1";

	public static final String COLUMN_FAMILY = "Standard1";

	public static void main(String[] args) {
		TSocket transport = new TSocket("localhost", 9160);
		TProtocol protocol = new TBinaryProtocol(transport);
		try {
			transport.open();
		} catch (TTransportException e) {
			throw new RuntimeException(e);
		}
		Cassandra.Client client = new Cassandra.Client(protocol);
		try {
			final String rowkey = "sample2";
			final long timestamp = System.currentTimeMillis();
			// ロウキーとカラムファミリのMapを作成
			Map<String, Map<String, List<Mutation>>> mutationMap = newMap();
			// カラムファミリのMapを作成。キーはカラムファミリ名、値はMutationのList
			Map<String, List<Mutation>> columnFamilyMap = newMap();

			// インサートの場合、MutationのListにはカラムかスーパーカラムを設定する
			List<Mutation> mutations = newList();
			// hogeというカラムキーで"hoge_value"を追加
			{
				Mutation mutation = new Mutation();
				ColumnOrSuperColumn csc = new ColumnOrSuperColumn()
						.setColumn(new Column("hoge".getBytes(), "hoge_value"
								.getBytes(), timestamp));
				mutation.setColumn_or_supercolumn(csc);
				mutations.add(mutation);
			}
			// fooというカラムキーで"foo_value"を追加
			{
				Mutation mutation = new Mutation();
				ColumnOrSuperColumn csc = new ColumnOrSuperColumn()
						.setColumn(new Column("foo".getBytes(), "foo_value"
								.getBytes(), timestamp));
				mutation.setColumn_or_supercolumn(csc);
				mutations.add(mutation);
			}
			// barというカラムキーで"bar_value"を追加
			{
				Mutation mutation = new Mutation();
				ColumnOrSuperColumn csc = new ColumnOrSuperColumn()
						.setColumn(new Column("bar".getBytes(), "bar_value"
								.getBytes(), timestamp));
				mutation.setColumn_or_supercolumn(csc);
				mutations.add(mutation);
			}
			columnFamilyMap.put(COLUMN_FAMILY, mutations);
			mutationMap.put(rowkey, columnFamilyMap);
			client.batch_mutate(KEYSPACE, mutationMap, ConsistencyLevel.ALL);
		} catch (InvalidRequestException e) {
			throw new RuntimeException(e);
		} catch (UnavailableException e) {
			throw new RuntimeException(e);
		} catch (TimedOutException e) {
			throw new RuntimeException(e);
		} catch (TException e) {
			throw new RuntimeException(e);
		} finally {
			try {
				transport.flush();
			} catch (TTransportException e) {
				throw new RuntimeException(e);
			} finally {
				transport.close();
			}
		}
		System.out.println("複数のカラムをまとめてインサート完了.");
	}

	private static <E> List<E> newList() {
		return new ArrayList<E>();
	}

	public static <K, V> Map<K, V> newMap() {
		return new HashMap<K, V>();
	}
}
