package examples;

import org.apache.cassandra.thrift.Cassandra;
import org.apache.cassandra.thrift.ColumnPath;
import org.apache.cassandra.thrift.ConsistencyLevel;
import org.apache.cassandra.thrift.InvalidRequestException;
import org.apache.cassandra.thrift.TimedOutException;
import org.apache.cassandra.thrift.UnavailableException;
import org.apache.thrift.TException;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransportException;

/**
 * シンプルなスーパーカラムのインサート。
 * 
 * @author shot
 * 
 */
public class SimpleSuperColumnInsert {

	public static final String KEYSPACE = "Keyspace1";

	public static final String COLUMN_FAMILY = "Super1";

	public static void main(String[] args) {
		TSocket transport = new TSocket("localhost", 9160);
		TProtocol protocol = new TBinaryProtocol(transport);
		try {
			transport.open();
		} catch (TTransportException e) {
			throw new RuntimeException(e);
		}
		Cassandra.Client client = new Cassandra.Client(protocol);
		try {
			final String key = "super_sample1";
			final String superColumnName = "sample1";
			final String columnName = "foo";
			final String value = "super_sample_value";
			final long timestamp = System.currentTimeMillis();

			// ColumnPathは単一のカラムの位置を特定するためのクラス
			final ColumnPath columnPath = new ColumnPath(COLUMN_FAMILY);
			// スーパーカラム名を指定
			columnPath.setSuper_column(superColumnName.getBytes());
			// カラム名を指定
			columnPath.setColumn(columnName.getBytes());

			// 1件カラムをインサート
			client.insert(KEYSPACE, key, columnPath, value.getBytes(),
					timestamp, ConsistencyLevel.ONE);
		} catch (InvalidRequestException e) {
			throw new RuntimeException(e);
		} catch (UnavailableException e) {
			throw new RuntimeException(e);
		} catch (TimedOutException e) {
			throw new RuntimeException(e);
		} catch (TException e) {
			throw new RuntimeException(e);
		} finally {
			try {
				transport.flush();
			} catch (TTransportException e) {
				throw new RuntimeException(e);
			} finally {
				transport.close();
			}
		}
		System.out.println("1件インサート完了.");
	}
}
